@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 
-/* Copyright (C) 2000-2004 Novell, Inc.
+/* Copyright (C) 2001-2004 Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU General Public
@@ -17,6 +17,8 @@
  * Boston, MA 02111-1307, USA.
  */
 
+/* camel-exchange-store.c: class for a exchange store */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -24,15 +26,15 @@
 #include <string.h>
 
 #include "e-util/e-path.h"
+#include <camel/camel-i18n.h>
 
 #include "camel-exchange-store.h"
 #include "camel-exchange-folder.h"
-#include <camel/camel-disco-diary.h>
 #include <camel/camel-session.h>
 
 #include <gal/util/e-util.h>
 
-static CamelDiscoStoreClass *parent_class = NULL;
+static CamelStoreClass *parent_class = NULL;
 
 #define CS_CLASS(so) ((CamelStoreClass *)((CamelObject *)(so))->klass)
 
@@ -45,25 +47,29 @@ static char  *get_name         (CamelService *service, gboolean brief);
 static CamelFolder     *get_trash       (CamelStore *store,
 					 CamelException *ex);
 
-static gboolean exchange_can_work_offline (CamelDiscoStore *disco_store);
-static gboolean exchange_connect_online (CamelService *service, CamelException *ex);
-static gboolean exchange_disconnect_online (CamelService *service, gboolean clean, CamelException *ex);
+static gboolean exchange_connect (CamelService *service, CamelException *ex);
+static gboolean exchange_disconnect (CamelService *service, gboolean clean, CamelException *ex);
 
 static CamelFolder *exchange_get_folder (CamelStore *store, const char *folder_name,
 					 guint32 flags, CamelException *ex);
 
-static CamelFolderInfo *exchange_get_folder_info_online (CamelStore *store, const char *top,
-							 guint32 flags, CamelException *ex);
+static CamelFolderInfo *exchange_get_folder_info (CamelStore *store, const char *top,
+						  guint32 flags, CamelException *ex);
+
+static CamelFolderInfo *exchange_create_folder (CamelStore *store,
+						const char *parent_name,
+						const char *folder_name,
+						CamelException *ex);
+static void             exchange_delete_folder (CamelStore *store,
+						const char *folder_name,
+						CamelException *ex);
+static void             exchange_rename_folder (CamelStore *store,
+						const char *old_name,
+						const char *new_name,
+						CamelException *ex);
 
 static void stub_notification (CamelObject *object, gpointer event_data, gpointer user_data);
 
-extern void mail_note_store (CamelStore *store, gpointer storage,
-			     gpointer corba_storage,
-			     void (*done) (CamelStore *store,
-					   CamelFolderInfo *info,
-					   void *data),
-			     void *data);
-
 static void
 class_init (CamelExchangeStoreClass *camel_exchange_store_class)
 {
@@ -71,36 +77,34 @@ class_init (CamelExchangeStoreClass *camel_exchange_store_class)
 		CAMEL_SERVICE_CLASS (camel_exchange_store_class);
 	CamelStoreClass *camel_store_class =
 		CAMEL_STORE_CLASS (camel_exchange_store_class);
-	CamelDiscoStoreClass *camel_disco_store_class =
-		CAMEL_DISCO_STORE_CLASS (camel_exchange_store_class);
 	
-	parent_class = CAMEL_DISCO_STORE_CLASS (camel_type_get_global_classfuncs (camel_disco_store_get_type ()));
+	parent_class = CAMEL_STORE_CLASS (camel_type_get_global_classfuncs (camel_store_get_type ()));
 	
 	/* virtual method overload */
 	camel_service_class->construct = construct;
 	camel_service_class->query_auth_types = query_auth_types;
 	camel_service_class->get_name = get_name;
+	camel_service_class->connect = exchange_connect;
+	camel_service_class->disconnect = exchange_disconnect;
 	
 	camel_store_class->get_trash = get_trash;
 	camel_store_class->free_folder_info = camel_store_free_folder_info_full;
-	
-	camel_disco_store_class->can_work_offline = exchange_can_work_offline;
-	camel_disco_store_class->connect_online = exchange_connect_online;
-	camel_disco_store_class->connect_offline = exchange_connect_online;
-	camel_disco_store_class->disconnect_online = exchange_disconnect_online;
-	camel_disco_store_class->disconnect_offline = exchange_disconnect_online;
-	camel_disco_store_class->get_folder_online = exchange_get_folder;
-	camel_disco_store_class->get_folder_offline = exchange_get_folder;
-	camel_disco_store_class->get_folder_resyncing = exchange_get_folder;
-	camel_disco_store_class->get_folder_info_online = exchange_get_folder_info_online;
-	camel_disco_store_class->get_folder_info_offline = exchange_get_folder_info_online;
-	camel_disco_store_class->get_folder_info_resyncing = exchange_get_folder_info_online;
+	camel_store_class->get_folder = exchange_get_folder;
+	camel_store_class->get_folder_info = exchange_get_folder_info;
+	camel_store_class->create_folder = exchange_create_folder;
+	camel_store_class->delete_folder = exchange_delete_folder;
+	camel_store_class->rename_folder = exchange_rename_folder;
 }
 
 static void
 init (CamelExchangeStore *exch, CamelExchangeStoreClass *klass)
 {
+	CamelStore *store = CAMEL_STORE (exch);
+
+	exch->folders_lock = g_mutex_new ();
 	exch->folders = g_hash_table_new (g_str_hash, g_str_equal);
+
+	store->flags &= ~(CAMEL_STORE_VTRASH | CAMEL_STORE_VJUNK);
 }
 
 static void
@@ -110,8 +114,8 @@ finalize (CamelExchangeStore *exch)
 		camel_object_unref (CAMEL_OBJECT (exch->stub));
 		exch->stub = NULL;
 	}
-	if (exch->note_idle)
-		g_source_remove (exch->note_idle);
+	if (exch->folders_lock)
+		g_mutex_free (exch->folders_lock);
 }
 
 CamelType
@@ -121,7 +125,7 @@ camel_exchange_store_get_type (void)
 
 	if (!camel_exchange_store_type) {
 		camel_exchange_store_type = camel_type_register (
-			CAMEL_DISCO_STORE_TYPE,
+			CAMEL_STORE_TYPE,
 			"CamelExchangeStore",
 			sizeof (CamelExchangeStore),
 			sizeof (CamelExchangeStoreClass),
@@ -153,6 +157,9 @@ construct (CamelService *service, CamelSession *session,
 	}
 
 	exch->storage_path = camel_session_get_storage_path (session, service, ex);
+
+	if (camel_url_get_param (url, "filter_junk"))
+		CAMEL_STORE (service)->flags |= CAMEL_STORE_VJUNK;
 }
 
 extern CamelServiceAuthType camel_exchange_password_authtype;
@@ -178,28 +185,13 @@ get_name (CamelService *service, gboolean brief)
 	}
 }
 
-static gboolean
-note_store (gpointer store)
-{
-	mail_note_store (store, NULL, NULL, NULL, NULL);
-	CAMEL_EXCHANGE_STORE (store)->note_idle = 0;
-	return FALSE;
-}
-
-static gboolean
-exchange_can_work_offline (CamelDiscoStore *disco_store)
-{
-	return FALSE;
-}
-
 #define EXCHANGE_STOREINFO_VERSION 1
 
 static gboolean
-exchange_connect_online (CamelService *service, CamelException *ex)
+exchange_connect (CamelService *service, CamelException *ex)
 {
 	CamelExchangeStore *store = CAMEL_EXCHANGE_STORE (service);
-	CamelDiscoStore *disco_store = CAMEL_DISCO_STORE (service);
-	char *real_user, *socket_path, *path;
+	char *real_user, *socket_path;
 	
 	if (!store->storage_path)
 		return FALSE;
@@ -231,25 +223,14 @@ exchange_connect_online (CamelService *service, CamelException *ex)
 		return FALSE;
 	}
 
-	path = g_strdup_printf ("%s/journal", store->storage_path);
-	disco_store->diary = camel_disco_diary_new (disco_store, path, ex);
-	g_free (path);
-	if (!disco_store->diary) {
-		camel_object_unref (CAMEL_OBJECT (store->stub));
-		store->stub = NULL;
-		return FALSE;
-	}
-	
 	camel_object_hook_event (CAMEL_OBJECT (store->stub), "notification",
 				 stub_notification, store);
 	
-	/* FIXME: Ick. Do something nicer in 1.2 */
-	store->note_idle = g_idle_add (note_store, service);
 	return TRUE;
 }
 
 static gboolean
-exchange_disconnect_online (CamelService *service, gboolean clean, CamelException *ex)
+exchange_disconnect (CamelService *service, gboolean clean, CamelException *ex)
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (service);
 
@@ -264,37 +245,43 @@ exchange_disconnect_online (CamelService *service, gboolean clean, CamelExceptio
 	return TRUE;
 }
 
+#define RETURN_VAL_IF_NOT_CONNECTED(service, ex, val) \
+	if (!camel_service_connect ((CamelService *)service, ex)) \
+		return val;
+
 static CamelFolder *
 exchange_get_folder (CamelStore *store, const char *folder_name,
 		     guint32 camel_flags, CamelException *ex)
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 	CamelFolder *folder;
-	char *name, *folder_dir;
+	char *folder_dir;
 
+	RETURN_VAL_IF_NOT_CONNECTED (store, ex, NULL);
+
+	g_mutex_lock (exch->folders_lock);
 	folder = g_hash_table_lookup (exch->folders, folder_name);
 	if (folder) {
 		/* This shouldn't actually happen, it should be caught
 		 * by the store-level cache...
 		 */
+		g_mutex_unlock (exch->folders_lock);
 		camel_object_ref (CAMEL_OBJECT (folder));
 		return folder;
 	}
 
 	folder = (CamelFolder *)camel_object_new (CAMEL_EXCHANGE_FOLDER_TYPE);
-	name = g_strdup (folder_name);
-	g_hash_table_insert (exch->folders, name, folder);
+	g_hash_table_insert (exch->folders, g_strdup (folder_name), folder);
+	g_mutex_unlock (exch->folders_lock);
 
 	folder_dir = e_path_to_physical (exch->storage_path, folder_name);
 	if (!camel_exchange_folder_construct (folder, store, folder_name,
-					      folder_dir, exch->stub, ex)) {
-		g_hash_table_remove (exch->folders, name);
-		g_free (name);
+					      camel_flags, folder_dir,
+					      exch->stub, ex)) {
 		g_free (folder_dir);
 		camel_object_unref (CAMEL_OBJECT (folder));
 		return NULL;
 	}
-
 	g_free (folder_dir);
 
 	/* If you move messages into a folder you haven't visited yet, it
@@ -311,6 +298,8 @@ get_trash (CamelStore *store, CamelException *ex)
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 
+	RETURN_VAL_IF_NOT_CONNECTED (store, ex, NULL);
+
 	if (!exch->trash_name) {
 		if (!camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_GET_TRASH_NAME,
 				      CAMEL_STUB_ARG_RETURN,
@@ -319,43 +308,76 @@ get_trash (CamelStore *store, CamelException *ex)
 			return NULL;
 	}
 
-	return CS_CLASS (store)->get_folder (store, exch->trash_name, 0, ex);
+	return camel_store_get_folder (store, exch->trash_name, 0, ex);
 }
 
 /* Note: steals @name and @uri */
 static CamelFolderInfo *
-make_folder_info (CamelExchangeStore *exch, char *name, char *uri)
+make_folder_info (CamelExchangeStore *exch, char *name, char *uri,
+		  int unread_count, int flags)
 {
 	CamelFolderInfo *info;
+	const char *path;
+
+	path = strstr (uri, "://");
+	if (!path)
+		return NULL;
+	path = strchr (path + 3, '/');
+	if (!path)
+		return NULL;
 
 	info = g_new0 (CamelFolderInfo, 1);
 	info->name = name;
-	info->url = uri;
+	info->uri = uri;
+	info->full_name = g_strdup (path + 1);
+	info->unread = unread_count;
 
-	info->path = strstr (uri, "://");
-	if (info->path) {
-		info->path = strchr (info->path + 3, '/');
-		if (info->path) {
-			info->path = g_strdup (info->path);
-			info->full_name = g_strdup (info->path + 1);
-			camel_url_decode (info->full_name);
-		}
-	}
-
-	info->unread_message_count = -1;
+	if (flags & CAMEL_STUB_FOLDER_NOSELECT)
+		info->flags = CAMEL_FOLDER_NOSELECT;
 
 	return info;
 }
 
 static CamelFolderInfo *
-exchange_get_folder_info_online (CamelStore *store, const char *top,
-				 guint32 flags, CamelException *ex)
+postprocess_tree (CamelFolderInfo *info)
+{
+	CamelFolderInfo *sibling;
+
+	if (info->child)
+		info->child = postprocess_tree (info->child);
+	if (info->next)
+		info->next = postprocess_tree (info->next);
+
+	/* If the node still has children, keep it */
+	if (info->child)
+		return info;
+	info->flags |= CAMEL_FOLDER_NOCHILDREN;
+
+	/* If it's a mail folder (not noselect), keep it */
+	if (!(info->flags & CAMEL_FOLDER_NOSELECT))
+		return info;
+
+	/* Otherwise delete it and return its sibling */
+	sibling = info->next;
+	info->next = NULL;
+	camel_folder_info_free (info);
+	return sibling;
+}
+			
+
+static CamelFolderInfo *
+exchange_get_folder_info (CamelStore *store, const char *top,
+			  guint32 flags, CamelException *ex)
 {
 	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
 	GPtrArray *folders, *folder_names, *folder_uris;
+	GArray *unread_counts;
+	GByteArray *folder_flags;
 	CamelFolderInfo *info;
 	int i;
 
+	RETURN_VAL_IF_NOT_CONNECTED (store, ex, NULL);
+
 	/* If the backend crashed, don't keep returning an error
 	 * each time auto-send/recv runs.
 	 */
@@ -363,9 +385,13 @@ exchange_get_folder_info_online (CamelStore *store, const char *top,
 		return NULL;
 
 	if (!camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_GET_FOLDER_INFO,
+			      CAMEL_STUB_ARG_STRING, top,
+			      CAMEL_STUB_ARG_UINT32, (flags & CAMEL_STORE_FOLDER_INFO_RECURSIVE) != 0,
 			      CAMEL_STUB_ARG_RETURN,
 			      CAMEL_STUB_ARG_STRINGARRAY, &folder_names,
 			      CAMEL_STUB_ARG_STRINGARRAY, &folder_uris,
+			      CAMEL_STUB_ARG_UINT32ARRAY, &unread_counts,
+			      CAMEL_STUB_ARG_BYTEARRAY, &folder_flags,
 			      CAMEL_STUB_ARG_END))
 		return NULL;
 	if (!folder_names) {
@@ -379,17 +405,73 @@ exchange_get_folder_info_online (CamelStore *store, const char *top,
 	folders = g_ptr_array_new ();
 	for (i = 0; i < folder_names->len; i++) {
 		info = make_folder_info (exch, folder_names->pdata[i],
-					 folder_uris->pdata[i]);
-		g_ptr_array_add (folders, info);
+					 folder_uris->pdata[i],
+					 g_array_index (unread_counts, int, i),
+					 folder_flags->data[i]);
+		if (info)
+			g_ptr_array_add (folders, info);
 	}
 	g_ptr_array_free (folder_names, TRUE);
 	g_ptr_array_free (folder_uris, TRUE);
+	g_array_free (unread_counts, TRUE);
+	g_byte_array_free (folder_flags, TRUE);
 
-	info = camel_folder_info_build (folders, NULL, '/', TRUE);
+	info = camel_folder_info_build (folders, top, '/', TRUE);
+	if (info)
+		info = postprocess_tree (info);
 	g_ptr_array_free (folders, TRUE);
+
 	return info;
 }
 
+static CamelFolderInfo *
+exchange_create_folder (CamelStore *store, const char *parent_name,
+			const char *folder_name, CamelException *ex)
+{
+	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
+	char *folder_uri;
+	guint32 unread_count, flags;
+	CamelFolderInfo *info;
+
+	if (!camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_CREATE_FOLDER,
+			      CAMEL_STUB_ARG_FOLDER, parent_name,
+			      CAMEL_STUB_ARG_STRING, folder_name,
+			      CAMEL_STUB_ARG_RETURN,
+			      CAMEL_STUB_ARG_STRING, &folder_uri,
+			      CAMEL_STUB_ARG_UINT32, &unread_count,
+			      CAMEL_STUB_ARG_UINT32, &flags,
+			      CAMEL_STUB_ARG_END))
+		return NULL;
+
+	info = make_folder_info (exch, g_strdup (folder_name),
+				 folder_uri, unread_count, flags);
+	info->flags |= CAMEL_FOLDER_NOCHILDREN;
+	return info;
+}
+
+static void
+exchange_delete_folder (CamelStore *store, const char *folder_name,
+			CamelException *ex)
+{
+	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
+
+	camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_DELETE_FOLDER,
+			 CAMEL_STUB_ARG_FOLDER, folder_name,
+			 CAMEL_STUB_ARG_END);
+}
+
+static void
+exchange_rename_folder (CamelStore *store, const char *old_name,
+			const char *new_name, CamelException *ex)
+{
+	CamelExchangeStore *exch = CAMEL_EXCHANGE_STORE (store);
+
+	camel_stub_send (exch->stub, ex, CAMEL_STUB_CMD_RENAME_FOLDER,
+			 CAMEL_STUB_ARG_FOLDER, old_name,
+			 CAMEL_STUB_ARG_FOLDER, new_name,
+			 CAMEL_STUB_ARG_END);
+}
+
 static void
 stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 {
@@ -411,7 +493,9 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		    camel_stub_marshal_decode_string (stub->status, &headers) == -1)
 			return;
 
+		g_mutex_lock (exch->folders_lock);
 		folder = g_hash_table_lookup (exch->folders, folder_name);
+		g_mutex_unlock (exch->folders_lock);
 		if (folder) {
 			camel_exchange_folder_add_message (folder, uid, flags,
 							   size, headers);
@@ -432,7 +516,9 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		    camel_stub_marshal_decode_string (stub->status, &uid) == -1)
 			return;
 
+		g_mutex_lock (exch->folders_lock);
 		folder = g_hash_table_lookup (exch->folders, folder_name);
+		g_mutex_unlock (exch->folders_lock);
 		if (folder)
 			camel_exchange_folder_remove_message (folder, uid);
 
@@ -450,7 +536,9 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		    camel_stub_marshal_decode_string (stub->status, &uid) == -1)
 			break;
 
+		g_mutex_lock (exch->folders_lock);
 		folder = g_hash_table_lookup (exch->folders, folder_name);
+		g_mutex_unlock (exch->folders_lock);
 		if (folder)
 			camel_exchange_folder_uncache_message (folder, uid);
 
@@ -470,7 +558,9 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		    camel_stub_marshal_decode_uint32 (stub->status, &flags) == -1)
 			break;
 
+		g_mutex_lock (exch->folders_lock);
 		folder = g_hash_table_lookup (exch->folders, folder_name);
+		g_mutex_unlock (exch->folders_lock);
 		if (folder)
 			camel_exchange_folder_update_message_flags (folder, uid, flags);
 
@@ -490,7 +580,9 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		    camel_stub_marshal_decode_string (stub->status, &value) == -1)
 			break;
 
+		g_mutex_lock (exch->folders_lock);
 		folder = g_hash_table_lookup (exch->folders, folder_name);
+		g_mutex_unlock (exch->folders_lock);
 		if (folder)
 			camel_exchange_folder_update_message_tag (folder, uid, name, value);
 
@@ -509,7 +601,9 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		if (camel_stub_marshal_decode_folder (stub->status, &folder_name) == -1)
 			break;
 
+		g_mutex_lock (exch->folders_lock);
 		folder = g_hash_table_lookup (exch->folders, folder_name);
+		g_mutex_unlock (exch->folders_lock);
 		if (folder)
 			camel_folder_freeze (folder);
 
@@ -525,7 +619,9 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		if (camel_stub_marshal_decode_folder (stub->status, &folder_name) == -1)
 			break;
 
+		g_mutex_lock (exch->folders_lock);
 		folder = g_hash_table_lookup (exch->folders, folder_name);
+		g_mutex_unlock (exch->folders_lock);
 		if (folder)
 			camel_folder_thaw (folder);
 
@@ -542,7 +638,8 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		    camel_stub_marshal_decode_string (stub->status, &uri) == -1)
 			break;
 
-		info = make_folder_info (exch, name, uri);
+		info = make_folder_info (exch, name, uri, -1, 0);
+		info->flags |= CAMEL_FOLDER_NOCHILDREN;
 		camel_object_trigger_event (CAMEL_OBJECT (exch),
 					    "folder_created", info);
 		camel_folder_info_free (info);
@@ -558,7 +655,7 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 		    camel_stub_marshal_decode_string (stub->status, &uri) == -1)
 			break;
 
-		info = make_folder_info (exch, name, uri);
+		info = make_folder_info (exch, name, uri, -1, 0);
 		camel_object_trigger_event (CAMEL_OBJECT (exch),
 					    "folder_deleted", info);
 		camel_folder_info_free (info);
@@ -567,7 +664,19 @@ stub_notification (CamelObject *object, gpointer event_data, gpointer user_data)
 
 	case CAMEL_STUB_RETVAL_FOLDER_RENAMED:
 	{
-		/* FIXME */
+		char *new_name, *new_uri;
+		CamelRenameInfo reninfo;
+
+		if (camel_stub_marshal_decode_folder (stub->status, &reninfo.old_base) == -1 ||
+		    camel_stub_marshal_decode_folder (stub->status, &new_name) == -1 ||
+		    camel_stub_marshal_decode_string (stub->status, &new_uri) == -1)
+			break;
+
+		reninfo.new = make_folder_info (exch, new_name, new_uri, -1, 0);
+		camel_object_trigger_event (CAMEL_OBJECT (exch),
+					    "folder_renamed", &reninfo);
+		camel_folder_info_free (reninfo.new);
+		g_free (reninfo.old_base);
 		break;
 	}
 
