@@ -19,6 +19,13 @@
 
 /* e2k-autoconfig: Automatic account configuration backend code */
 
+/* Note on gtk-doc: Several functions in this file have intentionally-
+ * broken gtk-doc comments (that have only a single "*" after the
+ * opening "/") so that they can be overridden by versions in
+ * docs/reference/tmpl/e2k-autoconfig.sgml that use better markup.
+ * If you change the docs here, be sure to change them there as well.
+ */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -349,6 +356,8 @@ get_ctx_auth_handler (SoupMessage *msg, gpointer user_data)
  *   %E2K_AUTOCONFIG_FAILED: Other error.
  *
  * Return value: the new context, or %NULL
+ *
+ * (If you change this comment, see the note at the top of this file.)
  **/
 E2kContext *
 e2k_autoconfig_get_context (E2kAutoconfig *ac, E2kOperation *op,
@@ -485,6 +494,10 @@ e2k_autoconfig_get_context (E2kAutoconfig *ac, E2kOperation *op,
 			goto done;
 		}
 
+		/* It's probably Exchange 2000... older versions
+		 * didn't include the MS-WebStorage header here. But
+		 * we don't know for sure.
+		 */
 		ac->version = E2K_EXCHANGE_UNKNOWN;
 	}
 
@@ -516,9 +529,14 @@ e2k_autoconfig_get_context (E2kAutoconfig *ac, E2kOperation *op,
 		goto done;
 	}
 
-	/* Make sure we didn't just get a page that wants to
-	 * redirect us to Exchange 5.5
-	 */
+	/* Make sure it's not Exchange 5.5 */
+	if (ac->version == E2K_EXCHANGE_UNKNOWN &&
+	    strstr (ac->owa_uri, "/logon.asp")) {
+		*result = E2K_AUTOCONFIG_EXCHANGE_5_5;
+		goto done;
+	}
+
+	/* Make sure it's not trying to redirect us to Exchange 5.5 */
 	for (node = doc->children; node; node = e2k_xml_find (node, "meta")) {
 		gboolean ex55 = FALSE;
 
@@ -539,15 +557,6 @@ e2k_autoconfig_get_context (E2kAutoconfig *ac, E2kOperation *op,
 		}
 	}
 
-	/* Don't go any further if it's not really Exchange 2000 */
-	if (ac->version == E2K_EXCHANGE_UNKNOWN) {
-		if (strstr (ac->owa_uri, "/logon.asp"))
-			*result = E2K_AUTOCONFIG_EXCHANGE_5_5;
-		else
-			*result = E2K_AUTOCONFIG_NO_OWA;
-		goto done;
-	}
-
 	/* Try to find the base URI */
 	node = e2k_xml_find (doc->children, "base");
 	if (node) {
@@ -598,6 +607,8 @@ static const int n_home_properties = sizeof (home_properties) / sizeof (home_pro
  *     e2k_autoconfig_check_exchange()
  *
  * Return value: an #E2kAutoconfigResult
+ *
+ * (If you change this comment, see the note at the top of this file.)
  **/
 E2kAutoconfigResult
 e2k_autoconfig_check_exchange (E2kAutoconfig *ac, E2kOperation *op)
@@ -649,6 +660,7 @@ e2k_autoconfig_check_exchange (E2kAutoconfig *ac, E2kOperation *op)
 		goto try_again;
 
 	case E2K_AUTOCONFIG_NO_OWA:
+	default:
 		/* If the provided OWA URI had no path, try appending
 		 * /exchange.
 		 */
@@ -663,9 +675,6 @@ e2k_autoconfig_check_exchange (E2kAutoconfig *ac, E2kOperation *op)
 		}
 		e2k_uri_free (euri);
 		return result;
-
-	default:
-		return result;
 	}
 
 	/* Find the link to the public folders */
@@ -805,7 +814,7 @@ find_global_catalog (E2kAutoconfig *ac)
 	return;
 }
 
-/*
+/**
  * e2k_autoconfig_get_global_catalog
  * @ac: an autoconfig context
  * @op: an #E2kOperation, for cancellation
@@ -853,6 +862,8 @@ e2k_autoconfig_get_global_catalog (E2kAutoconfig *ac, E2kOperation *op)
  *   %E2K_AUTOCONFIG_FAILED: Other error.
  *
  * Return value: an #E2kAutoconfigResult.
+ *
+ * (If you change this comment, see the note at the top of this file.)
  */
 E2kAutoconfigResult
 e2k_autoconfig_check_global_catalog (E2kAutoconfig *ac, E2kOperation *op)
@@ -1360,8 +1371,7 @@ read_config (void)
  * e2k_autoconfig_lookup_option:
  * @option: option name to look up
  *
- * Looks up an option in the config file. These values are NOT SIGNED.
- * The system administrator can put whatever values he wants here.
+ * Looks up an autoconfiguration hint in the config file (if present)
  *
  * Return value: the string value of the option, or %NULL if it is unset.
  **/
