@@ -39,12 +39,16 @@
 #include <e-util/e-passwords.h>
 #include <libedata-book/e-data-book-factory.h>
 #include <libedata-cal/e-data-cal-factory.h>
+#include <libedataserver/e-data-server-module.h>
 
 #include "e2k-utils.h"
 #include "addressbook/e-book-backend-exchange.h"
+#include "addressbook/e-book-backend-exchange-factory.h"
 #include "addressbook/e-book-backend-gal.h"
+#include "addressbook/e-book-backend-gal-factory.h"
 #include "calendar/e-cal-backend-exchange-calendar.h"
 #include "calendar/e-cal-backend-exchange-tasks.h"
+#include "calendar/e-cal-backend-exchange-factory.h"
 
 #include "exchange-autoconfig-wizard.h"
 #include "exchange-component.h"
@@ -93,12 +97,13 @@ setup_calendar_factory (void)
 		return FALSE;
 	}
 
-	e_data_cal_factory_register_method (
-		cal_factory, "exchange", ICAL_VEVENT_COMPONENT,
-		E_TYPE_CAL_BACKEND_EXCHANGE_CALENDAR);
-	e_data_cal_factory_register_method (
-		cal_factory, "exchange", ICAL_VTODO_COMPONENT,
-		E_TYPE_CAL_BACKEND_EXCHANGE_TASKS);
+	e_data_cal_factory_register_backend (cal_factory,
+					     (g_object_new (events_backend_exchange_factory_get_type(),
+						            NULL))); 
+
+	e_data_cal_factory_register_backend (cal_factory,
+					     (g_object_new (todos_backend_exchange_factory_get_type(),
+						            NULL))); 
 
 	/* register the factory with bonobo */
 	if (!e_data_cal_factory_register_storage (cal_factory, EXCHANGE_CALENDAR_FACTORY_ID)) {
@@ -122,14 +127,15 @@ static gboolean
 setup_addressbook_factory (void)
 {
         book_factory = e_data_book_factory_new ();
-
         if (!book_factory)
                 return FALSE;
 
-        e_data_book_factory_register_backend (
-                book_factory, "exchange", e_book_backend_exchange_new);
-	e_data_book_factory_register_backend (
-                book_factory, "gal", e_book_backend_gal_new);
+	e_data_book_factory_register_backend (book_factory, 
+					      (g_object_new (e_book_backend_exchange_factory_get_type(), 
+							     NULL)));
+	e_data_book_factory_register_backend (book_factory, 
+					      (g_object_new (e_book_backend_gal_factory_get_type(), 
+							     NULL)));
 
         g_signal_connect (book_factory, "last_book_gone",
 			  G_CALLBACK (last_book_gone_cb), NULL);
@@ -147,21 +153,17 @@ int
 main (int argc, char **argv)
 {
 	char *path;
-	char *config_directory;
 
-	bindtextdomain (GETTEXT_PACKAGE, CONNECTOR_LOCALEDIR);
-	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
-	textdomain (GETTEXT_PACKAGE);
+	bindtextdomain (PACKAGE, CONNECTOR_LOCALEDIR);
+	bind_textdomain_codeset (PACKAGE, "UTF-8");
+	textdomain (PACKAGE);
 
 	gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE, argc, argv,
 			    GNOME_PROGRAM_STANDARD_PROPERTIES,
 			    GNOME_PARAM_HUMAN_READABLE_NAME, _("Ximian Connector for Microsoft Exchange"),
 			    NULL);
 	e_icon_factory_init ();
-
-	config_directory = g_build_filename (g_get_home_dir(), ".evolution", NULL);
-	camel_init (config_directory, FALSE);
-	g_free(config_directory);
+	camel_init (NULL, FALSE);
 
 	path = g_strdup_printf ("/tmp/.exchange-%s", g_get_user_name ());
 	if (mkdir (path, 0700) == -1) {
